//
// Timer
// AVR ATtiny2313
// 2009.03.20
//
// Notice  Project-Configuration Option   Optimation -O0
//

#include <AVR/interrupt.h>
#include <avr/io.h>
#include <avr/wdt.h>
								//Timer mode
#define	MODE_OFF	0			//AC Off
#define	MODE_ON		1			//AC On
#define	MODE_TIMER	2			//Timer
#define	MODE_MAX	2

								//LED control
#define	LED_OFF		0			//LED Off
#define	LED_ON		1			//LED On
#define	LED_SLOW	2			//LED blink of slow
#define	LED_NORMAL	3			//LED blink of noraml
#define	LED_HI		4			//LED blink of hi

								//Port A
#define	DIPSW_SEL0	0			//W: DIP SW Selcet 0
#define	DIPSW_SEL1	1			//W: DIP SW Selcet 1

								//Port B
#define	DIPSW0		0			//R: DIP SW 0-3
#define	DIPSW1		1
#define	DIPSW2		2
#define	DIPSW3		3
#define	MODE_SW		4			//R: Mode SW

								//Port D
#define	AC			2			//W: AC  On/Off
#define	TIME_SW		5			//R: Time Scale 0:X1 1:X2
#define	LED			6			//W: LED On/Off

static uint32_t	ucnt   = 0;		//Timer Stik counter
static uint8_t	adjst1 = 0;		//Timer Adjstment counter 1
static uint8_t	adjst2 = 0;		//Timer Adjstment counter 2

static uint8_t	led_mode = 0;	//LED Mode
static uint8_t	led_cnt  = 0;	//LED Blink counter

void WatchDogTimerSet( void )
{
	WDTCSR = 0b00010000;
	WDTCSR = 0b00000000;
}

													//LED control
void Led_On ( void )								//LED On
{
	PORTD |= _BV( LED );
}

void Led_Off ( void )								//LED Off
{
	PORTD &= ~_BV( LED );
}

void Led_Blink ( void )								//LED Blink
{
	if (PORTD & 1<<LED ) Led_Off();
	else                 Led_On();
}

void LedModeSet( uint8_t mode )						//LED Set of mode
{
	led_mode = mode;
}

void LedControl(void)								//LED control (CTC Interrup)
{
	switch(	led_mode ) {							//Interval time 0.1mSec
	case LED_OFF:
		Led_Off();
		break;
	case LED_ON:
		Led_On();
		break;
	case LED_SLOW:
		led_cnt++;
		if ( led_cnt > 10 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	case LED_NORMAL:
		led_cnt++;
		if ( led_cnt > 5 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	case LED_HI:
		led_cnt++;
		if ( led_cnt > 3 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	}
}

													//Timer 0: Time control
ISR ( TIMER0_COMPA_vect )							//Interrupt Timer 0 CTC mode Clock 1MHz
{
	ucnt++;											// 0.1024 Sec
//	if ( adjst1++ >= 50 ) {							// Adjst of 0.02Sec
//		adjst1 = 0;
//		ucnt++;
//		if (adjst2++ >=5 ) {						// Adjst of 0.004Sec
//			ucnt++;
//			adjst2 = 0;
//		}
//	}
	LedControl();
}

void TimeInit( void )								//Initalize of Timer 0
{
	TCCR0B = 0b00000000;							//Stop to Timer 
	TCCR0A = 0b00000010;							//mode is CTC
	OCR0A  = 100;
	TCCR0B = 0b00000101;							//clock 1/1024

	TIMSK  = _BV(OCIE0A);							//Interrupt of comp mach A

	ucnt   = 0;										//stick counter
	adjst1 = 0;										//adjstment counter 1
	adjst2 = 0;										//adjstment counter 2
}

uint8_t TimeCheck( uint32_t start, uint32_t time )	//Timer check
{
	uint8_t sts =0;

	if (ucnt >= start) {
		if (time >= start) {
			if (ucnt >= time) sts = 1;
		}
	}
	else {
		if (time >= start)     sts = 1;
		else {
			if (ucnt >= time)  sts = 1;
		}
	} 
	return sts;
}

void TimeSet( uint32_t *start, uint32_t *time)		//Timer set
{
	*start = ucnt;
	*time  = *start + *time;
}

													//AC control
void AC_On ( void )									//AC power on
{
	PORTD |= _BV( AC );
}

void AC_Off ( void )								//AC power off
{
	PORTD &= ~_BV( AC );
}

													//Mode switch
uint8_t ModeSw()									//Read of mode switch
{
	if ( PINB & 1<<MODE_SW ) return 0;				// Active low
	else                     return 1;
}

uint8_t	TimeSw()									//Time scale switch
{													// 1: x2  2: x1
	if ( PIND & 1<<TIME_SW ) return 2;
	else					 return 1;
}
													//DIP switch
uint8_t DipSw( uint8_t select )						//Read of DIP switch
{
	uint8_t sts = 0;

	switch ( select ) {
	case 0:
		PORTA |=  _BV( DIPSW_SEL0 );
		PORTA &= ~_BV( DIPSW_SEL1 );
		sts = (PINB & 0x0f);
		break;
	case 1:
		PORTA &= ~_BV( DIPSW_SEL0 );
		PORTA |=  _BV( DIPSW_SEL1 );
		sts = (PINB & 0x0f);
		break;
	}

	return sts;
}

int main( void )
{
	uint32_t start, time;
	uint8_t	on_time, off_time;
	uint8_t	mode, on, sw, sw2, init;

								// Port Initialize
	DDRA  = 0b00000011;			// DIP SW Select 1-0
	DDRB  = 0b00000000;			//
	DDRD  = 0b01000100;			// LED, AC
	PORTA = 0b00000000;
	PORTB = 0b00010000;			// Pull Up Mode SW, DIPSW 0-3
	PORTD = 0b00100000;			// Time sw : Pull Up on

	TimeInit();

	sei();

	wdt_enable(WDTO_2S);

	AC_Off();
	Led_Off();

	on_time  = 8;
	off_time = 8;
	on       = 0;
	start    = 0;
	time     = 0;
	mode = MODE_TIMER;

	init = 1;
	sw   = ModeSw();
	sw2  = 0;
	while (1) {
		if ( sw2 == 0 ) {
			if ( sw == 1 || init == 1) {
				if ( ModeSw() || init == 1) {
					if ( init == 1 ) {
						init = 0;
					}
					else {
						mode ++;
						if ( mode > MODE_MAX ) mode = 0;
						sw2 = 1;
					}

					switch( mode ) {
					case MODE_OFF:
						mode = MODE_OFF;
						LedModeSet( LED_OFF );
						AC_Off();
						break;
					case MODE_ON:
						mode = MODE_ON;
						LedModeSet( LED_ON );
						AC_On();
						break;
					case MODE_TIMER:
						off_time = DipSw( 0 );
						on_time  = DipSw( 1 );
						if (off_time == 0) {
							mode = MODE_OFF;
							LedModeSet( LED_OFF );
							AC_Off();
							on = 0;
						}
						if (on_time  == 0) {
							mode = MODE_ON;
							LedModeSet( LED_ON );
							AC_On();
							on = 1;
						}
						if (mode == MODE_TIMER) {
							time = on_time * TimeSw() * 600;
							TimeSet( &start, &time );
							on = 1;
							LedModeSet( LED_NORMAL );
							AC_On();
						}
						break;
					}
				}
			}
			else sw = ModeSw();
		}
		else {
			if ( ModeSw() == 0 ) sw = sw2 = 0;
		}

		switch( mode ) {
		case MODE_TIMER: 
			if (TimeCheck( start, time)) {
				if ( on ) {
					time = off_time * TimeSw() * 600;
					on = 0;
					LedModeSet( LED_SLOW );
					AC_Off();
				}
				else {
					time = on_time * TimeSw() * 600;
					on = 1;
					LedModeSet( LED_NORMAL );
					AC_On();
				}
				TimeSet( &start, &time );
			}
			break;
		case MODE_ON:
			break;
		case MODE_OFF:
			break;
		}
		wdt_reset();
	}
}


